@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -327,7 +327,7 @@ evtcbnode_s *EvtcbQue::schedule_once(evtcb_pf cb, evtcbhead_t *session,
 void EvtcbQue::recycle(evtcbnode_s *pObj)
 {
     logState("recycle()", pObj);
-    memset(pObj, 0, sizeof(evtcbnode_s));
+    memset((void *) pObj, 0, sizeof(evtcbnode_s));
     ls_atomic_spin_lock(&s_poolLock);
     s_pCbnodePool->recycle(pObj);
     ls_atomic_spin_unlock(&s_poolLock);
