@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -50,7 +50,8 @@ public:
     int appendEnv(const char *pEnv, int len);
     int buildReqHeader(int uid, int gid, int priority, int umaskVal,
                        const char *pChroot, int chrootLen,
-                       const char *pReal, int pathLen, const RLimits *pLimits);
+                       const char *pReal, int pathLen, const RLimits *pLimits,
+                       int dropCaps);
     int finalize(int req_id, const char *pSecret, int type);
 
     LS_NO_COPY_ASSIGN(CgidReq);
