@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -135,6 +135,10 @@ void ClientInfo::release()
         m_pLocInfo = NULL;
     }
 #endif
+
+    //FIXME: this may cause memory leak if it holds unreleased objects.
+    //       but, it is better than reusing it with wrong data.
+    m_moduleData.clear();
 }
 
 
