@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -18,7 +18,6 @@
 #ifndef CONTEXTNODE_H
 #define CONTEXTNODE_H
 
-
 #include <util/hashstringmap.h>
 
 #define HTA_UNKNOWN         0
@@ -27,16 +26,33 @@
 
 class HttpContext;
 class ContextList;
+class AutoStr2;
+class StringList;
+class HttpHandler;
+class ContextNode;
 
-class ContextNode : public HashStringMap< ContextNode * >
+class ChildNodeList : public HashStringMap< ContextNode * >
 {
+public:
+    ChildNodeList()
+        : HashStringMap< ContextNode * >(7)
+    {}
+    ~ChildNodeList()
+    {   release_objects();  }
 
+};
+
+
+class ContextNode
+{
+    ChildNodeList      *m_pChildren;
     ContextNode        *m_pParentNode;
     HttpContext        *m_pContext;
     char               *m_pLabel;
     long                m_lHTALastCheck;
     short               m_iHTAState;
-    short               m_iRelease;
+    char                m_isDir;
+    char                m_iRelease;
 
     ContextNode(const ContextNode &rhs);
     void operator=(const ContextNode &rhs);
@@ -45,54 +61,73 @@ public:
 
     ContextNode(const char *pchLabel, ContextNode *pParentNode);
 
-    const HttpContext *getContext() const  {   return m_pContext ;     }
-    HttpContext *getContext()              {   return m_pContext ;     }
-    ContextNode *getParentNode() const     {   return m_pParentNode ;  }
+    const HttpContext *getContext() const   {   return m_pContext ;     }
+    HttpContext *getContext()               {   return m_pContext ;     }
+    ContextNode *getParentNode() const      {   return m_pParentNode ;  }
+    ChildNodeList *getChildren() const      {   return m_pChildren;     }
     HttpContext *getParentContext();
     void setChildrenParentContext(const HttpContext *pOldParent,
                                   const HttpContext *pNewParent, int noUpdateRedirect);
     const char *getLabel() const            {   return m_pLabel ;       }
     void setLabel(const char *l);
 
-    void removeRedirectContext(HttpContext *pParent);
     void contextInherit(const HttpContext *pRootContext);
 
-    void setRelease(short r)              {   m_iRelease = r;         }
-    short getRelease() const                {   return m_iRelease;      }
+    void setRelease(char r)                 {   m_iRelease = r;         }
+    char getRelease() const                 {   return m_iRelease;      }
 
-    void setHTAState(char state)          {   m_iHTAState = state;    }
+    void setIsDir(char dir)                 {   m_isDir = dir;          }
+    char isDir() const                      {   return m_isDir;         }
+
+    void setHTAState(char state)            {   m_iHTAState = state;    }
     short getHTAState()  const              {   return m_iHTAState;     }
 
     long getLastCheck() const               {   return m_lHTALastCheck; }
-    void setLastCheck(long check)         {   m_lHTALastCheck = check;}
+    void setLastCheck(long check)           {   m_lHTALastCheck = check;}
 
-    void setContext(HttpContext *pContext) {  m_pContext = pContext;  }
+    void setContext(HttpContext *pContext)  {  m_pContext = pContext;  }
     void setContextUpdateParent(HttpContext *pContext, int noUpdateRedirect);
     ContextNode *insertChild(const char *pchLabel);
     ContextNode *match(const char *pStart);
+    ContextNode *find(const char *pStart);
+
     ContextNode()
-        : HashStringMap< ContextNode * >(10)
-        , m_pLabel(NULL)
+        : m_pLabel(NULL)
         , m_lHTALastCheck(0)
         , m_iHTAState(HTA_UNKNOWN)
+        , m_isDir(0)
+        , m_iRelease(0)
     {}
 
     ~ContextNode();
+
+    ContextNode *findChild(const char *pStart)
+    {
+        ChildNodeList::iterator iter ;
+        if (m_pChildren)
+        {
+            iter = m_pChildren->find(pStart);
+            if (iter != m_pChildren->end())
+                return iter.second();
+        }
+        return NULL;
+    }
+    void removeChild(ContextNode *pChild);
+
     ContextNode *getChildNode(const char *pStart, long lastCheck = 0)
     {
-        iterator iter ;
-        iter = find(pStart);
-        if (iter == end())
-        {
-            ContextNode *pNode = insertChild(pStart);
-            if (pNode)
-                pNode->setLastCheck(lastCheck);
+
+        ContextNode *pNode = findChild(pStart);
+        if (pNode)
             return pNode;
-        }
-        else
-            return iter.second();
+        pNode = insertChild(pStart);
+        if (pNode && lastCheck > 0)
+            pNode->setLastCheck(lastCheck);
+        return pNode;
     }
-    void getAllContexts(ContextList &list);
+    //void getAllContexts( ContextList &list );
+    const HttpContext *matchIndexes(
+        const StringList *pIndexList, AutoStr2 *&pIdx);
 };
 
 #endif
