@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -20,13 +20,12 @@
 
 
 #include <http/expiresctrl.h>
-
+#include <util/autostr.h>
 #include <sys/types.h>
 #include <stddef.h>
 
 #define MAX_MIME_LEN 256
 
-class AutoStr2;
 class HttpHandler;
 class MimeMap;
 class MIList;
@@ -39,6 +38,14 @@ class XmlNodeList;
 class HttpVHost;
 class ConfigCtx;
 
+typedef struct _scriptHanlderData {
+    char* suffix;
+    char *handler;
+    char *type;
+    int state;
+} scriptHanlderData;
+
+
 class MimeSetting
 {
     AutoStr2           *m_psMIME;
@@ -122,8 +129,22 @@ public:
     static int  isValidMimeType(const char *pDescr);
     static int  shouldKeepAlive(const char *pMIME);
     static int getExtAppGUid(const XmlNode* pExtAppNode, uid_t &udi, gid_t &gid);
+
+    static void releaseHandlerData(scriptHanlderData *pData, int dataCount);
+    static void mergeHandlerList(ConfigCtx& currentCtx,
+                                 scriptHanlderData *parentData,
+                                 int parentDataCount,
+                                 const XmlNodeList  *pList,
+                                 scriptHanlderData *pdata,
+                                 int *count);
+    static int _configScriptHandler(ConfigCtx& currentCtx, const char *value,
+                                    AutoStr *psuffix, AutoStr *psType,
+                                    char *achHandler, int max_len);
+    static int getSuffixCount(const XmlNodeList *pList);
     static int configScriptHandler(const XmlNodeList *pList,
-                                   HttpMime *pHttpMime, HttpVHost *vhost);
+                                  HttpMime *pHttpMime,
+                                  HttpVHost *vhost);
+
     static void addMimeHandler(const HttpHandler *pHdlr, char *pMime,
                                HttpMime *pHttpMime,
                                const char *pSuffix);
