@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -558,17 +558,14 @@ int StaticFileCacheData::tryCreateCompressed(char useBrotli)
     close(fd);
     if (size < 409600)
     {
-
         long ret = compressFile(useBrotli);
         if (ret == -1)
             LS_WARN("Failed to compress file %s, file size %ld!",
                     m_real.c_str(), (long)size);
-
         *p = 'l';
         unlink(pPath->buf());
         *p = 0;
         return ret;
-
     }
     else
     {
@@ -592,7 +589,7 @@ int StaticFileCacheData::tryCreateCompressed(char useBrotli)
         *p = 'l';
         unlink(pPath->buf());
         *p = 0;
-        exit(1);
+        _exit(1);
     }
 }
 
@@ -635,7 +632,7 @@ int StaticFileCacheData::compressFile(char useBrotli)
 
     GzipBuf gzBuf;
     Compressor *pCompressor;
-    VMemBuf compressedFile;
+    VMemBuf compressBuf;
     int iCompressLevel;
 
 #ifdef USE_BROTLI
@@ -666,24 +663,28 @@ int StaticFileCacheData::compressFile(char useBrotli)
             return LS_FAIL;
     }
 
+    if (compressBuf.set(VMBUF_ANON_MAP, 8192) == LS_FAIL)
+        return LS_FAIL;
+
     char achFileName[4096];
     snprintf(achFileName, 4096, "%s.XXXXXX", pPath->c_str());
     int fd = mkstemp(achFileName);
-    ret = compressedFile.setFd(achFileName, fd);
-    if (ret)
+    if (fd == -1)
+        return LS_FAIL;
+
+    pCompressor->setCompressCache(&compressBuf);
+    if (pCompressor->beginStream())
     {
         close(fd);
-        return ret;
-    }
-    pCompressor->setCompressCache(&compressedFile);
-    if (pCompressor->beginStream())
         return LS_FAIL;
+    }
     off_t offset = 0;
     int len;
     off_t wanted;
     const char *pData;
     char achBuf[8192];
-    while (true)
+    ret = 0;
+    while (ret == 0)
     {
         wanted = getFileSize() - offset;
         if (wanted <= 0)
@@ -694,18 +695,24 @@ int StaticFileCacheData::compressFile(char useBrotli)
             len = 8192;
         pData = m_fileData.getCacheData(offset, wanted, achBuf, len);
         if (wanted <= 0)
-            return LS_FAIL;
-        if (pCompressor->write(pData, wanted) == LS_FAIL)
-            return LS_FAIL;
+            ret = LS_FAIL;
+        else if (pCompressor->write(pData, wanted) == LS_FAIL)
+            ret = LS_FAIL;
         offset += wanted;
-
+        if (compressBuf.getCurWOffset() >= 8192)
+        {
+            if (compressBuf.writeToFile(fd) == LS_FAIL)
+                ret = LS_FAIL;
+            pCompressor->resetCompressCache();
+        }
     }
-    if (0 == pCompressor->endStream())
+    if (ret == 0 && 0 == pCompressor->endStream())
     {
         off_t size;
-        if (compressedFile.exactSize(&size) == 0)
+        if (compressBuf.writeToFile(fd) == 0)
         {
-            compressedFile.close();
+            size = lseek(fd, (size_t)0, SEEK_CUR);
+            close(fd);
             unlink(pPath->buf());
             rename(achFileName, pPath->buf());
 
@@ -717,6 +724,7 @@ int StaticFileCacheData::compressFile(char useBrotli)
             return size;
         }
     }
+    close(fd);
     return LS_FAIL;
 }
 
@@ -863,12 +871,30 @@ int StaticFileCacheData::readyCompressed(char compressMode)
         }
     }
 
-    if (compressMode & SFCD_MODE_BROTLI)
+    if ((compressMode & SFCD_MODE_BROTLI) && s_iBrCompressLevel == 0)
+        compressMode &= ~SFCD_MODE_BROTLI;
+
+    if (compressMode & SFCD_MODE_BROTLI) // brotli active AND not valid
     {
         statBr = ls_fio_stat(m_bredPath.c_str(), &stBr);
         LS_DBG_H("readyCompressed() path %s statBr %d",
-                 m_bredPath.c_str(), statBr);
+                m_bredPath.c_str(), statBr);
+        if ((statBr == -1) || (stBr.st_mtime != getLastMod()))
+        {
+            // update br
+            if ((statBr = compressHelper(m_bredPath, m_pBrotli, stBr, statBr, 1)))
+            {
+                LS_DBG_H("readyCompressed compress br error %s.",
+                         m_bredPath.c_str());
+                compressMode &= ~SFCD_MODE_BROTLI;
+            }
+            else
+                compressMode &= ~SFCD_MODE_GZIP;
+        }
+        else
+            compressMode &= ~SFCD_MODE_GZIP;
     }
+
     if (compressMode & SFCD_MODE_GZIP)
     {
         statGz = ls_fio_stat(m_gzippedPath.c_str(), &stGzip);
@@ -876,43 +902,12 @@ int StaticFileCacheData::readyCompressed(char compressMode)
                 || (stGzip.st_mtime != getLastMod()));
         LS_DBG_H("readyCompressed() path %s statGz %d retGz %d",
                  m_gzippedPath.c_str(), statGz, retGz);
-    }
-
-    if (compressMode & SFCD_MODE_BROTLI) // brotli active AND not valid
-    {
-        if ((statBr != -1) && (stBr.st_mtime == getLastMod()))
-        {
-            // use br
-        }
-        else if (!(compressMode & SFCD_MODE_GZIP) || retGz)
-        {
-            // update br
-            if ((statBr = compressHelper(m_bredPath, m_pBrotli, stBr, statBr, 1)))
-            {
-                LS_DBG_H("readyCompressed compress br error %s.",
-                         m_bredPath.c_str());
-                return LS_FAIL;
-            }
-        }
-        else
-        {
-            compressMode &= ~SFCD_MODE_BROTLI;
-        }
-    }
-    else if (compressMode & SFCD_MODE_GZIP)
-    {
         if (retGz && (statGz = compressHelper(m_gzippedPath, m_pGzip, stGzip, statGz, 0)))
         {
             LS_DBG_H("readyCompressed() compress gzip error %s or file size not suitable for gzip.",
                     m_gzippedPath.c_str());
             return LS_FAIL;
         }
-        // else use gz
-    }
-    else
-    {
-        LS_ERROR("Compress with both Brotli and Gzip turned off.");
-        return LS_FAIL;
     }
 
     if ((compressMode & SFCD_MODE_BROTLI)
