@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * HEADERS stream logic
  */
@@ -10,9 +10,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/queue.h>
-#ifdef MSVC
-#include <vc_compat.h>
-#endif
 
 #include "lsquic.h"
 #include "lsquic_types.h"
@@ -81,6 +78,7 @@ static lsquic_stream_ctx_t *
 headers_on_new_stream (void *stream_if_ctx, lsquic_stream_t *stream)
 {
     struct headers_stream *hs = stream_if_ctx;
+
     lshpack_dec_init(&hs->hs_hdec);
     if (0 != lshpack_enc_init(&hs->hs_henc))
     {
@@ -292,18 +290,8 @@ headers_on_error (void *ctx, lsquic_stream_id_t stream_id,
     struct headers_stream *hs = ctx;
     switch (err)
     {
-    case FR_ERR_DUPLICATE_PSEH:
-    case FR_ERR_INCOMPL_REQ_PSEH:
-    case FR_ERR_UNNEC_REQ_PSEH:
-    case FR_ERR_BAD_REQ_HEADER:
-    case FR_ERR_INCOMPL_RESP_PSEH:
-    case FR_ERR_UNNEC_RESP_PSEH:
-    case FR_ERR_UNKNOWN_PSEH:
-    case FR_ERR_UPPERCASE_HEADER:
-    case FR_ERR_MISPLACED_PSEH:
-    case FR_ERR_MISSING_PSEH:
+    case FR_ERR_BAD_HEADER:
     case FR_ERR_DECOMPRESS:
-    case FR_ERR_HEADERS_TOO_LARGE:
     case FR_ERR_SELF_DEP_STREAM:
         LSQ_INFO("error %u is a stream error (stream %"PRIu64")", err,
                                                                     stream_id);
@@ -313,8 +301,8 @@ headers_on_error (void *ctx, lsquic_stream_id_t stream_id,
     case FR_ERR_NONZERO_STREAM_ID:
     case FR_ERR_UNEXPECTED_PUSH:
     case FR_ERR_ZERO_STREAM_ID:
-    case FR_ERR_NOMEM:
     case FR_ERR_EXPECTED_CONTIN:
+    case FR_ERR_OTHER_ERROR:
         LSQ_INFO("error %u is a connection error (stream %"PRIu64")", err,
                                                                     stream_id);
         hs->hs_callbacks->hsc_on_conn_error(hs->hs_cb_ctx);
@@ -385,7 +373,6 @@ headers_on_settings (void *ctx, uint16_t setting_id, uint32_t setting_value)
 int
 lsquic_headers_stream_push_promise (struct headers_stream *hs,
         lsquic_stream_id_t stream_id64, lsquic_stream_id_t promised_stream_id64,
-        const struct iovec *path, const struct iovec *host,
         const struct lsquic_http_headers *headers)
 {
     uint32_t stream_id = stream_id64;
@@ -394,7 +381,7 @@ lsquic_headers_stream_push_promise (struct headers_stream *hs,
     LSQ_DEBUG("promising stream %u in response to stream %u",
                                             promised_stream_id, stream_id);
     s = lsquic_frame_writer_write_promise(hs->hs_fw, stream_id,
-                                    promised_stream_id, path, host, headers);
+                                                promised_stream_id, headers);
     if (0 == s)
     {
         lsquic_stream_wantwrite(hs->hs_stream,
