@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_logger.h -- logging functions and macros.
  *
@@ -51,7 +51,6 @@ enum lsquic_logger_module {
     LSQLM_EVENT,
     LSQLM_ENGINE,
     LSQLM_CONN,
-    LSQLM_RECHIST,
     LSQLM_STREAM,
     LSQLM_PARSE,
     LSQLM_CFCW,
@@ -63,6 +62,7 @@ enum lsquic_logger_module {
     LSQLM_HSK_ADAPTER,
     LSQLM_BBR,
     LSQLM_CUBIC,
+    LSQLM_ADAPTIVE_CC,
     LSQLM_HEADERS,
     LSQLM_FRAME_WRITER,
     LSQLM_FRAME_READER,
@@ -70,6 +70,7 @@ enum lsquic_logger_module {
     LSQLM_TOKGEN,
     LSQLM_ENG_HIST,
     LSQLM_SPI,
+    LSQLM_HPI,
     LSQLM_DI,
     LSQLM_PRQ,
     LSQLM_PACER,
@@ -85,6 +86,8 @@ enum lsquic_logger_module {
     LSQLM_QPACK_DEC,
     LSQLM_PRIO,
     LSQLM_BW_SAMPLER,
+    LSQLM_PACKET_RESIZE,
+    LSQLM_CONN_STATS,
     N_LSQUIC_LOGGER_MODULES
 };
 
@@ -248,8 +251,10 @@ lsquic_logger_lopt (const char *optarg);
 
 #define CID_FMT ".*s"
 
-#define CID_BITS(cid) 2 * (int) (cid)->len, \
-                                    (lsquic_cid2str(cid, cidbuf_), cidbuf_)
+#define CID_BITS_B(cid, b) 2 * (int) (cid)->len, \
+                                    (lsquic_cid2str(cid, b), b)
+
+#define CID_BITS(cid) CID_BITS_B(cid, cidbuf_)
 
 void
 lsquic_cid2str (const struct lsquic_cid *, char *out);
