@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <stdint.h>
 #include <stdlib.h>
 
@@ -8,7 +8,7 @@
 
 
 lsquic_packno_t
-restore_packno (lsquic_packno_t cur_packno,
+lsquic_restore_packno (lsquic_packno_t cur_packno,
                 unsigned len,
                 lsquic_packno_t max_packno)
 {
