@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_parse_common.h
  */
@@ -6,6 +6,10 @@
 #ifndef LSQUIC_PARSE_COMMON_H
 #define LSQUIC_PARSE_COMMON_H 1
 
+#ifdef WIN32
+#include "vc_compat.h"
+#endif
+
 struct lsquic_packet_in;
 struct packin_parse_state;
 
