@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_parse_ietf_v1.c -- Parsing functions specific to IETF QUIC v1
  */
@@ -49,6 +49,8 @@
 #define FRAME_TYPE_ACK_FREQUENCY    0xAF
 #define FRAME_TYPE_TIMESTAMP        0x2F5
 
+#define MIN(a, b) ((a) < (b) ? (a) : (b))
+
 static int
 ietf_v1_gen_one_varint (unsigned char *, size_t, unsigned char, uint64_t);
 
@@ -120,8 +122,7 @@ ietf_v1_packout_header_size_long_by_packet (const struct lsquic_conn *lconn,
 
 
 static size_t
-ietf_v1_packout_header_size_short (const struct lsquic_conn *lconn,
-                                enum packet_out_flags flags, size_t dcid_len)
+ietf_v1_packout_header_size_short (enum packet_out_flags flags, size_t dcid_len)
 {
     enum packno_bits bits;
     size_t sz;
@@ -138,19 +139,13 @@ ietf_v1_packout_header_size_short (const struct lsquic_conn *lconn,
 
 static size_t
 ietf_v1_packout_max_header_size (const struct lsquic_conn *lconn,
-                                enum packet_out_flags flags, size_t dcid_len)
+    enum packet_out_flags flags, size_t dcid_len, enum header_type header_type)
 {
-    if (lconn->cn_flags & LSCONN_HANDSHAKE_DONE)
-        return ietf_v1_packout_header_size_short(lconn, flags, dcid_len);
-    else if (lconn->cn_flags & LSCONN_SERVER)
-        /* Server does not set the token in its Initial packet header: set
-         * the packet type to something else in order not to overestimate
-         * header size.
-         */
-        return ietf_v1_packout_header_size_long_by_flags(lconn, HETY_HANDSHAKE,
-                                                            flags, dcid_len);
+    if ((lconn->cn_flags & LSCONN_HANDSHAKE_DONE)
+                                                && header_type == HETY_NOT_SET)
+        return ietf_v1_packout_header_size_short(flags, dcid_len);
     else
-        return ietf_v1_packout_header_size_long_by_flags(lconn, HETY_INITIAL,
+        return ietf_v1_packout_header_size_long_by_flags(lconn, header_type,
                                                             flags, dcid_len);
 }
 
@@ -192,7 +187,7 @@ write_packno (unsigned char *p, lsquic_packno_t packno,
 static int
 gen_long_pkt_header (const struct lsquic_conn *lconn,
             const struct lsquic_packet_out *packet_out, unsigned char *buf,
-                                                                size_t bufsz)
+            size_t bufsz, unsigned *packno_off_p, unsigned *packno_len_p)
 {
     unsigned payload_len, bits;
     enum packno_bits packno_bits;
@@ -242,6 +237,8 @@ gen_long_pkt_header (const struct lsquic_conn *lconn,
     vint_write(p, payload_len, bits, 1 << bits);
     p += 1 << bits;
 
+    *packno_off_p = p - buf;
+    *packno_len_p = iquic_packno_bits2len(packno_bits);
     p += write_packno(p, packet_out->po_packno, packno_bits);
 
     return p - buf;
@@ -251,7 +248,7 @@ gen_long_pkt_header (const struct lsquic_conn *lconn,
 static int
 gen_short_pkt_header (const struct lsquic_conn *lconn,
             const struct lsquic_packet_out *packet_out, unsigned char *buf,
-                                                                size_t bufsz)
+            size_t bufsz, unsigned *packno_off_p, unsigned *packno_len_p)
 {
     unsigned packno_len, cid_len, need;
     enum packno_bits packno_bits;
@@ -264,7 +261,7 @@ gen_short_pkt_header (const struct lsquic_conn *lconn,
     if (need > bufsz)
         return -1;
 
-    buf[0] = 0x40
+    buf[0] = QUIC_BIT
            | (lsquic_packet_out_spin_bit(packet_out) << 5)
            | (lsquic_packet_out_square_bit(packet_out) << 4)
            | (lsquic_packet_out_loss_bit(packet_out) << 3)
@@ -277,6 +274,8 @@ gen_short_pkt_header (const struct lsquic_conn *lconn,
 
     (void) write_packno(buf + 1 + cid_len, packet_out->po_packno, packno_bits);
 
+    *packno_off_p = 1 + cid_len;
+    *packno_len_p = packno_len;
     return need;
 }
 
@@ -284,38 +283,14 @@ gen_short_pkt_header (const struct lsquic_conn *lconn,
 static int
 ietf_v1_gen_reg_pkt_header (const struct lsquic_conn *lconn,
             const struct lsquic_packet_out *packet_out, unsigned char *buf,
-                                                                size_t bufsz)
+            size_t bufsz, unsigned *packno_off, unsigned *packno_len)
 {
     if (packet_out->po_header_type == HETY_NOT_SET)
-        return gen_short_pkt_header(lconn, packet_out, buf, bufsz);
+        return gen_short_pkt_header(lconn, packet_out, buf, bufsz, packno_off,
+                                                                    packno_len);
     else
-        return gen_long_pkt_header(lconn, packet_out, buf, bufsz);
-}
-
-
-static void
-ietf_v1_packno_info (const struct lsquic_conn *lconn,
-        const struct lsquic_packet_out *packet_out, unsigned *packno_off,
-        unsigned *packno_len)
-{
-    unsigned token_len; /* Need intermediate value to quiet compiler warning */
-
-    if (packet_out->po_header_type == HETY_NOT_SET)
-        *packno_off = 1 +
-            (packet_out->po_flags & PO_CONN_ID ? packet_out->po_path->np_dcid.len : 0);
-    else
-        *packno_off = 1
-                    + 4
-                    + 1
-                    + packet_out->po_path->np_dcid.len
-                    + 1
-                    + CN_SCID(lconn)->len
-                    + (packet_out->po_header_type == HETY_INITIAL ?
-                        (token_len = packet_out->po_token_len,
-                            (1 << vint_val2bits(token_len)) + token_len) : 0)
-                    + 2;
-    *packno_len = iquic_packno_bits2len(
-        lsquic_packet_out_packno_bits(packet_out));
+        return gen_long_pkt_header(lconn, packet_out, buf, bufsz, packno_off,
+                                                                    packno_len);
 }
 
 
@@ -327,7 +302,7 @@ ietf_v1_packout_size (const struct lsquic_conn *lconn,
 
     if ((lconn->cn_flags & LSCONN_HANDSHAKE_DONE)
                                 && packet_out->po_header_type == HETY_NOT_SET)
-        sz = ietf_v1_packout_header_size_short(lconn, packet_out->po_flags,
+        sz = ietf_v1_packout_header_size_short(packet_out->po_flags,
                                             packet_out->po_path->np_dcid.len);
     else
         sz = ietf_v1_packout_header_size_long_by_packet(lconn, packet_out);
@@ -353,6 +328,10 @@ ietf_v1_gen_stream_frame (unsigned char *buf, size_t buf_len,
     unsigned slen, olen, dlen;
     unsigned char *p = buf + 1;
 
+#if _MSC_VER
+    obits = 0, dbits = 0;
+#endif
+
     assert(!!fin ^ !!size);
 
     /* We do not check that stream_id, offset, and size are smaller
@@ -439,14 +418,15 @@ ietf_v1_gen_stream_frame (unsigned char *buf, size_t buf_len,
 
 int
 lsquic_ietf_v1_gen_crypto_frame (unsigned char *buf, unsigned char first_byte,
-            size_t buf_len, uint64_t offset, size_t size, gcf_read_f gcf_read,
-            void *stream)
+            size_t buf_len, lsquic_stream_id_t UNUSED_1, uint64_t offset,
+            int UNUSED_2, size_t size, gsf_read_f gsf_read, void *stream)
 {
     unsigned char *const end = buf + buf_len;
     unsigned char *p;
     unsigned obits, dbits;
     unsigned olen, dlen;
     size_t nr, n_avail;
+    int dummy_fin;
 
     obits = vint_val2bits(offset);
     olen = 1 << obits;
@@ -466,7 +446,7 @@ lsquic_ietf_v1_gen_crypto_frame (unsigned char *buf, unsigned char first_byte,
     vint_write(p, offset, obits, olen);
     p += olen;
 
-    nr = gcf_read(stream, p + dlen, size);
+    nr = gsf_read(stream, p + dlen, size, &dummy_fin);
     assert(nr != 0);    /* This indicates error in the caller */
     assert(nr <= size); /* This also indicates an error in the caller */
 
@@ -479,10 +459,43 @@ lsquic_ietf_v1_gen_crypto_frame (unsigned char *buf, unsigned char first_byte,
 
 static int
 ietf_v1_gen_crypto_frame (unsigned char *buf, size_t buf_len,
-        uint64_t offset, size_t size, gcf_read_f gcf_read, void *stream)
+        lsquic_stream_id_t stream_id, uint64_t offset, int fin,
+        size_t size, gsf_read_f gsf_read, void *stream)
 {
-    return lsquic_ietf_v1_gen_crypto_frame(buf, 0x6, buf_len, offset,
-                                                size, gcf_read, stream);
+    return lsquic_ietf_v1_gen_crypto_frame(buf, 0x6, buf_len, stream_id,
+                                        offset, fin, size, gsf_read, stream);
+}
+
+
+static int
+ietf_v1_dec_stream_frame_size (unsigned char *p, size_t new_size)
+{
+    /* 0b00001XXX
+     *  0x4     OFF
+     *  0x2     LEN
+     *  0x1     FIN
+     */
+    unsigned bits;
+
+    const char type = *p++;
+    if (!(type & 0x2))
+        return 1;
+
+    /* Stream ID */
+    bits = *p >> 6;
+    p += 1 << bits;
+
+    if (type & 0x4)
+    {
+        /* Offset */
+        bits = *p >> 6;
+        p += 1 << bits;
+    }
+
+    /* Write new size */
+    bits = *p >> 6;
+    vint_write(p, new_size, bits, 1 << bits);
+    return 0;
 }
 
 
@@ -711,7 +724,7 @@ ietf_v1_rst_frame_size (lsquic_stream_id_t stream_id, uint64_t error_code,
 
 static int
 ietf_v1_gen_rst_frame (unsigned char *buf, size_t buf_len,
-        lsquic_stream_id_t stream_id, uint64_t error_code, uint64_t final_size)
+        lsquic_stream_id_t stream_id, uint64_t final_size, uint64_t error_code)
 {
     unsigned vbits;
     unsigned char *p;
@@ -1030,7 +1043,7 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
         rsize = range->high - range->low;
         a = vint_val2bits(gap - 1);
         b = vint_val2bits(rsize);
-        if (ecn_needs + (1 << a) + (1 << b) > AVAIL())
+        if (ecn_needs + (1 << a) + (1 << b) > (unsigned)AVAIL())
             break;
         if (addl_ack_blocks == VINT_MAX_ONE_BYTE)
         {
@@ -1055,7 +1068,7 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
 
     if (ecn_counts)
     {
-        assert(ecn_needs <= AVAIL());
+        assert(ecn_needs <= (unsigned)AVAIL());
         for (ecn = 1; ecn <= 3; ++ecn)
         {
             vint_write(p, ecn_counts[ecnmap[ecn]], bits[ecnmap[ecn]], 1 << bits[ecnmap[ecn]]);
@@ -1185,7 +1198,7 @@ ietf_v1_parse_path_resp_frame (const unsigned char *buf, size_t len,
 }
 
 
-void
+static void
 ietf_v1_turn_on_fin (unsigned char *stream_frame_header)
 {
     *stream_frame_header |= 1;
@@ -1609,7 +1622,7 @@ ietf_v1_new_token_frame_size (size_t token_sz)
 }
 
 
-int
+static int
 ietf_v1_gen_new_token_frame (unsigned char *buf, size_t buf_sz,
                                 const unsigned char *token, size_t token_sz)
 {
@@ -1649,7 +1662,7 @@ ietf_v1_new_connection_id_frame_size (unsigned seqno, unsigned scid_len)
 }
 
 
-int
+static int
 ietf_v1_gen_new_connection_id_frame (unsigned char *buf, size_t buf_sz,
             unsigned seqno, const struct lsquic_cid *cid,
             const unsigned char *token, size_t token_sz)
@@ -1686,6 +1699,22 @@ static const enum header_type bits2ht[4] =
 };
 
 
+#if LSQUIC_QIR
+/* Return true if the parsing function is to enforce the minimum DCID
+ * length requirement as specified in IETF v1 and the I-Ds.
+ */
+static int
+enforce_initial_dcil (lsquic_ver_tag_t tag)
+{
+    enum lsquic_version version;
+
+    version = lsquic_tag2ver(tag);
+    return version != (enum lsquic_version) -1
+        && ((1 << version) & LSQUIC_IETF_VERSIONS);
+}
+#endif
+
+
 int
 lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
                 size_t length, int is_server, unsigned cid_len,
@@ -1738,6 +1767,14 @@ lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
     switch (header_type)
     {
     case HETY_INITIAL:
+#if LSQUIC_QIR
+        if (!enforce_initial_dcil(tag))
+        {
+            /* Count even zero-length DCID as having DCID */
+            packet_in->pi_flags |= PI_CONN_ID;
+        }
+        else
+#endif
         if (is_server && dcil < MIN_INITIAL_DCID_LEN)
             return -1;
         r = vint_read(p, end, &token_len);
@@ -2003,7 +2040,7 @@ lsquic_ietf_v1_gen_ver_nego_pkt (unsigned char *buf, size_t bufsz,
     if (need > bufsz)
         return -1;
 
-    *buf++ = 0x80 | 0x40 | rand;
+    *buf++ = 0x80 | QUIC_BIT | rand;
     memset(buf, 0, 4);
     buf += 4;
 
@@ -2060,20 +2097,38 @@ ietf_v1_parse_handshake_done_frame (const unsigned char *buf, size_t buf_len)
 
 static int
 ietf_v1_gen_ack_frequency_frame (unsigned char *buf, size_t buf_len,
-    uint64_t seqno, uint64_t pack_tol, uint64_t upd_mad)
+    uint64_t seqno, uint64_t pack_tol, uint64_t upd_mad, int ignore)
 {
-    return ietf_v1_gen_frame_with_varints(buf, buf_len, 4,
+    int sz;
+
+    sz = ietf_v1_gen_frame_with_varints(buf, buf_len, 4,
         (uint64_t[]){ FRAME_TYPE_ACK_FREQUENCY, seqno, pack_tol, upd_mad });
+    if (sz > 0 && (size_t) sz < buf_len)
+    {
+        buf[sz++] = !!ignore;
+        return sz;
+    }
+    else
+        return -1;
 }
 
 
 static int
 ietf_v1_parse_ack_frequency_frame (const unsigned char *buf, size_t buf_len,
-    uint64_t *seqno, uint64_t *pack_tol, uint64_t *upd_mad)
+    uint64_t *seqno, uint64_t *pack_tol, uint64_t *upd_mad, int *ignore)
 {
-    return ietf_v1_parse_frame_with_varints(buf, buf_len,
+    int sz;
+
+    sz = ietf_v1_parse_frame_with_varints(buf, buf_len,
                 FRAME_TYPE_ACK_FREQUENCY,
                 3, (uint64_t *[]) { seqno, pack_tol, upd_mad });
+    if (sz > 0 && (size_t) sz < buf_len && buf[sz] < 2)
+    {
+        *ignore = buf[sz++];
+        return sz;
+    }
+    else
+        return -1;
 }
 
 
@@ -2081,7 +2136,7 @@ static unsigned
 ietf_v1_ack_frequency_frame_size (uint64_t seqno, uint64_t pack_tol,
     uint64_t upd_mad)
 {
-    return ietf_v1_frame_with_varints_size(4,
+    return 1 + ietf_v1_frame_with_varints_size(4,
             (uint64_t[]){ FRAME_TYPE_ACK_FREQUENCY, seqno, pack_tol, upd_mad });
 }
 
@@ -2111,6 +2166,82 @@ ietf_v1_parse_timestamp_frame (const unsigned char *buf, size_t buf_len,
 }
 
 
+static int
+ietf_v1_parse_datagram_frame (const unsigned char *buf, size_t buf_len,
+                                        const void **data, size_t *data_len)
+{
+    uint64_t len;
+    int s;
+
+    /* Length and frame type have been checked already */
+    assert(buf_len > 0);
+    assert(buf[0] == 0x30 || buf[0] == 0x31);
+
+    if (buf[0] & 1)
+    {
+        s = vint_read(buf + 1, buf + buf_len, &len);
+        if (s > 0 && 1 + s + len <= buf_len)
+        {
+            *data = buf + 1 + s;
+            *data_len = len;
+            return 1 + s + len;
+        }
+        else
+            return -1;
+    }
+    else
+    {
+        *data = buf + 1;
+        *data_len = buf_len - 1;
+        return buf_len;
+    }
+}
+
+
+static unsigned
+ietf_v1_datagram_frame_size (size_t sz)
+{
+    return 1u + vint_size(sz) + sz;
+}
+
+
+static int
+ietf_v1_gen_datagram_frame (unsigned char *buf, size_t bufsz, size_t min_sz,
+    size_t max_sz,
+    ssize_t (*user_callback)(struct lsquic_conn *, void *, size_t),
+    struct lsquic_conn *lconn)
+{
+    unsigned bits, len_sz;
+    ssize_t nw;
+
+    /* We always generate length.  A more efficient implementation would
+     * complicate the API.
+     */
+    if (min_sz)
+        bits = vint_val2bits(min_sz);
+    else
+        bits = vint_val2bits(bufsz);
+    len_sz = 1u << bits;
+
+    if (1 + len_sz + min_sz > bufsz)
+    {
+        errno = ENOBUFS;
+        return -1;
+    }
+
+    nw = user_callback(lconn, buf + 1 + len_sz, min_sz ? min_sz
+                                            : MIN(bufsz - 1 - len_sz, max_sz));
+    if (nw >= 0)
+    {
+        buf[0] = 0x31;
+        vint_write(&buf[1], (uint64_t) nw, bits, len_sz);
+        return 1 + len_sz + nw;
+    }
+    else
+        return -1;
+}
+
+
 const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
 {
     .pf_gen_reg_pkt_header            =  ietf_v1_gen_reg_pkt_header,
@@ -2118,6 +2249,7 @@ const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
     .pf_gen_stream_frame              =  ietf_v1_gen_stream_frame,
     .pf_calc_stream_frame_header_sz   =  ietf_v1_calc_stream_frame_header_sz,
     .pf_parse_stream_frame            =  ietf_v1_parse_stream_frame,
+    .pf_dec_stream_frame_size         =  ietf_v1_dec_stream_frame_size,
     .pf_parse_ack_frame               =  ietf_v1_parse_ack_frame,
     .pf_gen_ack_frame                 =  ietf_v1_gen_ack_frame,
     .pf_gen_blocked_frame             =  ietf_v1_gen_blocked_frame,
@@ -2142,7 +2274,6 @@ const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
     .pf_parse_path_resp_frame         =  ietf_v1_parse_path_resp_frame,
     .pf_calc_packno_bits              =  ietf_v1_calc_packno_bits,
     .pf_packno_bits2len               =  ietf_v1_packno_bits2len,
-    .pf_packno_info                   =  ietf_v1_packno_info,
     .pf_gen_crypto_frame              =  ietf_v1_gen_crypto_frame,
     .pf_parse_crypto_frame            =  ietf_v1_parse_crypto_frame,
     .pf_calc_crypto_frame_header_sz   =  ietf_v1_calc_crypto_frame_header_sz,
@@ -2181,4 +2312,7 @@ const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
     .pf_ack_frequency_frame_size      =  ietf_v1_ack_frequency_frame_size,
     .pf_gen_timestamp_frame           =  ietf_v1_gen_timestamp_frame,
     .pf_parse_timestamp_frame         =  ietf_v1_parse_timestamp_frame,
+    .pf_parse_datagram_frame          =  ietf_v1_parse_datagram_frame,
+    .pf_gen_datagram_frame            =  ietf_v1_gen_datagram_frame,
+    .pf_datagram_frame_size           =  ietf_v1_datagram_frame_size,
 };
