@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_qdec_hdl.h -- QPACK decoder streams handler
  *
@@ -14,6 +14,7 @@ struct lsquic_conn;
 struct lsquic_stream;
 struct lsquic_stream_if;
 struct lsquic_engine_public;
+struct qpack_exp_record;
 
 
 struct qpack_dec_hdl
@@ -22,6 +23,8 @@ struct qpack_dec_hdl
     enum {
         QDH_INITIALIZED     = 1 << 0,
         QDH_PUSH_PROMISE    = 1 << 1,
+        QDH_SAVE_UA         = 1 << 2,
+        QDH_SERVER          = 1 << 3,
     }                        qdh_flags;
     struct lsqpack_dec       qdh_decoder;
     struct lsquic_stream    *qdh_enc_sm_in;
@@ -33,6 +36,8 @@ struct qpack_dec_hdl
     void                    *qdh_hsi_ctx;
     void                   (*qdh_on_dec_sent_func)(void *);
     void                    *qdh_on_dec_sent_ctx;
+    struct qpack_exp_record *qdh_exp_rec;
+    char                    *qdh_ua;
 };
 
 int
@@ -62,16 +67,19 @@ enum lsqpack_read_header_status
 lsquic_qdh_header_in_continue (struct qpack_dec_hdl *, struct lsquic_stream *,
                                 const unsigned char **, size_t);
 
-void
-lsquic_qdh_unref_stream (struct qpack_dec_hdl *, struct lsquic_stream *);
-
 void
 lsquic_qdh_cancel_stream (struct qpack_dec_hdl *,
                                                 struct lsquic_stream *);
 
+void
+lsquic_qdh_cancel_stream_id (struct qpack_dec_hdl *, lsquic_stream_id_t);
+
 int
 lsquic_qdh_arm_if_unsent (struct qpack_dec_hdl *, void (*)(void *), void *);
 
+const char *
+lsquic_qdh_get_ua (const struct qpack_dec_hdl *);
+
 extern const struct lsquic_stream_if *const lsquic_qdh_enc_sm_in_if;
 extern const struct lsquic_stream_if *const lsquic_qdh_dec_sm_out_if;
 
