@@ -74,9 +74,9 @@ typedef struct
 
 class LsShmPool : public ls_shmpool_s
 {
+    friend class LsShm;
 public:
     LsShmPool();
-    ~LsShmPool();
 
 public:
     LsShmHash *getNamedHash(const char *name, LsShmSize_t init_size,
@@ -247,6 +247,9 @@ private:
     LsShmPool &operator=(const LsShmPool &other);
     bool operator==(const LsShmPool &other);
 
+    //NOTE: make it private to disallow direct call, only use LsShmPool->close().
+    ~LsShmPool();
+
     uint32_t            m_iMagic;
     LsShm              *m_pShm;         // SHM handle
     LsShmStatus_t       m_status;       // Ready ...
