@@ -1,8 +1,20 @@
-/*
- * Copyright 2002 Lite Speed Technologies Inc, All Rights Reserved.
- * LITE SPEED PROPRIETARY/CONFIDENTIAL.
- */
-
+/*****************************************************************************
+*    Open LiteSpeed is an open source HTTP server.                           *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
+*                                                                            *
+*    This program is free software: you can redistribute it and/or modify    *
+*    it under the terms of the GNU General Public License as published by    *
+*    the Free Software Foundation, either version 3 of the License, or       *
+*    (at your option) any later version.                                     *
+*                                                                            *
+*    This program is distributed in the hope that it will be useful,         *
+*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
+*    GNU General Public License for more details.                            *
+*                                                                            *
+*    You should have received a copy of the GNU General Public License       *
+*    along with this program. If not, see http://www.gnu.org/licenses/.      *
+*****************************************************************************/
 
 #include <sslpp/sslasyncpk.h>
 
@@ -94,6 +106,12 @@ static ssl_private_key_result_t AsyncPrivateKeySign(
     }
 
     data->m_pkey = SSL_get_privatekey(ssl);
+    if (NULL == data->m_pkey)
+    {
+        ERROR_MESSAGE("[SSL: %p] Key is NULL!\n", ssl);
+        return ssl_private_key_failure;
+    }
+
     if (EVP_PKEY_id(data->m_pkey) !=
             SSL_get_signature_algorithm_key_type(signature_algorithm))
     {
